typedef struct {
  unsigned long dword0;
  unsigned long dword1;
} segment_desc;

// Our interrupt routine for the keyboard
extern void keyb_int();

void init_interrupts()
{
  // The IDT containing interrupt addresses and flags
  segment_desc idt[0x100];

  // The IDT descriptor, containing address and size of the IDT
  unsigned long idt_desc[2];

  // Help variables
  unsigned long idt_address;
  unsigned long keyb_address;


  // Send ICW1
  out(0x20, 0x11);
  out(0xA0, 0x11);

  // Specify interrupt numbers
  out(0x21, 0x20);      // PIC1 = 0x20-0x27
  out(0xA1, 0x28);      // PIC2 = 0x28-0x2F

  // Set pin to communicate on - DON'T CHANGE!
  out(0x21, 0x4);
  out(0xA1, 0x2);

  // Send ICW4
  out(0x21, 0x1);
  out(0xA1, 0x1);

  // Mask interrupts
  out(0x21, 0xFD);      // Enable keyboard interrupt on PIC1
  out(0xA1, 0xFF);      // We don't need any interrupts from PIC2


  keyb_address = (unsigned long)keyb_int;
  // Lower 16 bits of the interrupt address
  // and it's segment (offset 0x18 in the GDT)
  idt[0x21].dword0 = (keyb_address & 0xFFFF) | (0x18 << 16);

  // Higher 16 bits of the interrupt address
  // and the interrupt settings (look in Tutorial 4 for details)
  idt[0x21].dword1 = (keyb_address & 0xFFFF0000) | 0x8E00;


  idt_address = (unsigned long)idt;

  // Specify the limit (size) and the offset of the IDT
  idt_desc[0] = 0x800 + ((idt_address & 0xFFFF) << 16);
  idt_desc[1] = idt_address >> 16;


  // Load the IDT descriptor and enable interrupts
  asm("lidt %0\n"
      "sti"
      :"=m"(idt_desc));
}
